from pathlib import Path
from typing import Annotated

import typer

from blprint.modules.blueprints.collect_blueprint_values import (
    collect_blueprint_variables,
)
from blprint.modules.blueprints.format_blueprint_variables import (
    format_blueprint_variables,
)
from blprint.modules.blueprints.get_all_blueprint_codes import get_all_blueprint_codes
from blprint.modules.blueprints.instantiate_blueprint import instantiate_blueprint
from blprint.modules.templates.get_templates_from_blueprint import (
    get_templates_from_blueprint,
)

app = typer.Typer()


@app.command()
def create(
    blueprint_name: str,
    destination: Path,
    blueprint_folder_path: Annotated[Path, typer.Argument()] = Path("./blueprints"),
):
    if not destination.exists():
        raise typer.BadParameter(f"Destination {destination} does not exist")

    if not destination.is_dir():
        raise typer.BadParameter(f"Destination {destination} is not a directory")

    if not blueprint_folder_path.exists():
        raise typer.BadParameter(
            f"Blueprint path {blueprint_folder_path} does not exist"
        )

    blueprint_path: Path = Path(str(blueprint_folder_path) + "/" + blueprint_name)

    templates: list[str] = get_templates_from_blueprint(blueprint_path, blueprint_name)

    all_blueprint_codes: list[str] = get_all_blueprint_codes(templates, blueprint_path)

    blueprint_variable_names: list[str] = [
        code.split("__")[0] for code in all_blueprint_codes
    ]

    unique_blueprint_variable_names = list(set(blueprint_variable_names))

    blueprint_variables = collect_blueprint_variables(unique_blueprint_variable_names)

    formatted_blueprint_variables = format_blueprint_variables(
        all_blueprint_codes, blueprint_variables
    )

    instantiate_blueprint(
        templates, blueprint_path, destination, formatted_blueprint_variables
    )



from pathlib import Path

from blprint.modules.templates.get_template_variables import get_template_variables


def get_all_blueprint_codes(templates: list[str], blueprint_path: Path) -> list[str]:
    variables = set()

    for template in templates:
        template_variables = get_template_variables(template, blueprint_path)
        variables.update(template_variables)

    return list(variables)



from pathlib import Path

from jinja2 import Environment, FileSystemLoader, meta


def get_template_variables(template: str, blueprint_path: Path) -> set:
    env = Environment(loader=FileSystemLoader(blueprint_path))
    template_source = env.loader.get_source(env, template)
    parsed_content = env.parse(str(template_source))
    return meta.find_undeclared_variables(parsed_content)


import os
from pathlib import Path

from blprint.modules.templates.format_string import format_string
from blprint.modules.templates.instantiate_template import instantiate_template


def instantiate_blueprint(
        templates: list[str],
        blueprint_path: Path,
        destination_path: Path,
        template_variables: dict[str, str]
):
    for template in templates:
        template_content: str = instantiate_template(template, blueprint_path, template_variables)
        renamed_template = template.replace(".template", '')
        renamed_template = format_string(renamed_template, template_variables)

        template_instance_path: str = os.path.join(destination_path, renamed_template)

        with open(template_instance_path, 'w', encoding='utf-8') as file:
            file.write(template_content)



import typer

from blprint.modules.blueprints.commands.create.create_command import app as create_app

app = typer.Typer()

app.add_typer(create_app)

def main():
    app()

if __name__ == "__main__":
    app()


import stringcase

letter_case_functions = {
    "snake_case": lambda x: stringcase.snakecase(x),
    "camel_case": lambda x: stringcase.camelcase(x),
    "pascal_case": lambda x: stringcase.pascalcase(x),
}


def format_blueprint_variables(
    blueprint_variable_codes: list[str], blueprint_variables: dict[str, str]
):
    formatted_blueprint_variables = {}

    for code in blueprint_variable_codes:
        splitted_code = code.split("__")
        name = splitted_code[0]

        if len(splitted_code) > 2 or len(splitted_code) == 0:
            raise (Exception(f"The code {code} is not a valid code"))

        if len(splitted_code) == 1:
            formatted_blueprint_variables[code] = blueprint_variables[name]
            continue

        letter_case = splitted_code[1]

        case_function = letter_case_functions.get(letter_case, None)

        if not case_function:
            raise (Exception(f"The code {code} is not a valid code"))

        formatted_blueprint_variables[code] = case_function(blueprint_variables[name])

    return formatted_blueprint_variables



from pathlib import Path


def get_templates_from_blueprint(blueprint_path: Path, blueprint: str) -> list[str]:

    if not blueprint_path.exists():
        raise ValueError(f"Blueprint {blueprint} does not exist")

    if not blueprint_path.is_dir():
        raise ValueError(f"Blueprint {blueprint} is not a directory")

    templates: list[str] = [path.name for path in blueprint_path.iterdir()]

    return templates


def collect_blueprint_variables(variable_names: list[str]):
    variables = {}

    for variable_name in variable_names:
        value = input(f"Value for '{variable_name}': ")
        variables[variable_name] = value

    return variables



from pathlib import Path
from jinja2 import Environment, FileSystemLoader

def instantiate_template(template: str, blueprint_path: Path, kwargs: dict) -> str:
    env = Environment(loader=FileSystemLoader(blueprint_path))
    template = env.get_template(template)
    rendered_str = template.render(**kwargs)
    return rendered_str